/*
 * Project name:
      AlphaNum Click
 * Copyright:
      (c) mikroElektronika, 2015.
 * Revision History:
      20150423:
      - Initial release (PT);
 * Description:
      AlphaNum click is a simple solution for adding 14-segment alphanumeric displays to your device.
      The board carries two TLC5926 16-bit Constant-Current LED sink Drivers as well as a dual character LED 14-segment display.
      AlphaNum click communicates with the target board through mikroBUS RST, CS, SCK, MISO, MOSI, PWM and INT pins
      (marked here as LE2, LE1, CLK, DOUT, DIN, NUMSEL and NUMSEL# respectively).
      The board is designed to use either a 3.3V or 5V power supply. While a standard 7-segment display is sufficient only for
      displaying digits, a 14-segment display (also known as a starburst display) is necessary for legibly rendering the ISO
      basic Latin alphabet. The dual character display onboard AlphaNum click also contain two commas
      (decimal points if used for digits) as additional segments.
      The two TLC5926 ICs contain 16-bit shift registers and data latches to convert serial input data into parallel output format.

      Code writes text, special characters and numbers on 14-seg displays.

 * Test configuration:
     MCU:             STM32F407VG
                      http://www.st.com/web/catalog/mmc/FM141/SC1169/SS1031/LN1564/PF221020
     Dev. Board:      EasyMx v7 for STM32(R) ARM(R)
                      http://www.mikroe.com/easymx-pro/stm32/
     Oscillator:      XT-PLL, 120.000MHz
     ext. modules:    AlphaNum click
                      http://www.mikroe.com/click/alphanum-g/
                      http://www.mikroe.com/click/alphanum-r/
     SW:              mikroC PRO for ARM
                      http://www.mikroe.com/mikroc/arm/
 * NOTES:
     - Place AlphaNum click board in the mikroBUS socket 1.
 */

// AlphaNum Click module connections
sbit TLC5926_LE2  at GPIOC_ODR.B2;         // Set LE2 on PC2
sbit TLC5926_LE1  at GPIOD_ODR.B13;        // Set LE1 on PD13
sbit TLC5926_NS   at GPIOA_ODR.B0;         // Set NS  on PA0
sbit TLC5926_nNS  at GPIOD_ODR.B10;        // Set NS# on PD10
// AlphaNum Click module connections

unsigned temp = 0;
unsigned int pwm_period1;

short int cnt = 0;
unsigned int wait_time, current_duty;

const unsigned CharTable[51]={

 // 14segment bit order
 // (a)(b)(c)(d)(e)(f)(i)(j)(g1)(g2)(k)(m)(I)(h)(dp)

 // SPECIAL CHARACTERS
 //-------------------------------------------------
 // Hex        Symbol            Binary
 //-------------------------------------------------
 // 0x136D     $                 0b00001001101101101
 // 0x2809     SIGMA             0b00010100000001001
 // 0x3CC0     Cyrilic letter    0b00011110011000000

 // CHARACTERS
 //-------------------------------------------------
 // Hex        Symbol            Binary
 //-------------------------------------------------
    0x0300,    // '-'            0b0000001100000000
    0x4000,    // '.'            0b0100000000000000
    0x2176,    // '/'            0b0000100010000000
    0x003F,    // '0'            0b0000000000111111
    0x0006,    // '1'            0b0000000000000110
    0x031B,    // '2'            0b0000001100011011
    0x030F,    // '3'            0b0000001100001111
    0x0000,    // '4'            0b0000000001000000
    0x032D,    // '5'            0b0000001100101101
    0x033D,    // '6'            0b0000001100111101
    0x0007,    // '7'            0b0000000000000111
    0x033F,    // '8'            0b0000001100111111
    0x032F,    // '9'            0b0000001100101111
    0x1040,    // ':'            0b0001000001000000
    0x0840,    // ';'            0b0000100001000000
    0x0480,    // '<'            0b0000010010000000
    0x0776,    // '='            0b0000001100001000
    0x2800,    // '>'            0b0010100000000000
    0x1223,    // '?'            0b0001001000100011
    0x027B,    // '@'            0b0000001001111011
    0x0337,    // 'A'            0b0000001100110111
    0x124F,    // 'B'            0b0001001001001111
    0x0039,    // 'C'            0b0000001100111001
    0x104F,    // 'D'            0b0001000001001111
    0x0339,    // 'E'            0b0000001100111001
    0x0331,    // 'F'            0b0000001100110001
    0x023D,    // 'G'            0b0000001000111101
    0x0336,    // 'H'            0b0000001100110110
    0x1040,    // 'I'            0b0001000001000000
    0x001E,    // 'J'            0b0000000000011110
    0x05B0,    // 'K'            0b0000010110110000
    0x0038,    // 'L'            0b0000000000111000
    0x20B6,    // 'M'            0b0010000010110110
    0x2436,    // 'N'            0b0010010000110110
    0x003F,    // 'O'            0b0000000000111111
    0x0333,    // 'P'            0b0000001100110011
    0x443F,    // 'Q'            0b0100010000111111
    0x0733,    // 'R'            0b0000011100110011
    0x032D,    // 'S'            0b0000001100101101
    0x1063,    // 'T'            0b0001000001100011
    0x003E,    // 'U'            0b0000000000111110
    0x08B0,    // 'V'            0b0000100010110000
    0x0C36,    // 'W'            0b0000110000110110
    0x2C80,    // 'X'            0b0010110010000000
    0x3080,    // 'Y'            0b0011000010000000
    0x0889,    // 'Z'            0b0000100010001001
    0x0039,    // '['            0b0000000000111001
    0x0880,    // '/'            0b0000100010000000
    0x000F,    // ']'            0b0000000000001111
    0x0081,    // '^'            0b0000000010000001
    0x0008,    // '_'            0b0000000000001000
};

unsigned intro_text[10]={
  0x003F, 0x003F,
  0x003F, 0x003F,
  0x003F, 0x003F,
  0x003F, 0x003F,
  0x003F, 0x003F
};

void WriteSegments(unsigned a, unsigned b){
  PWM_TIM2_Stop(_PWM_CHANNEL1);
  GPIO_Digital_Output(&GPIOD_ODR,  _GPIO_PINMASK_13);    // Set LE1
  GPIO_Digital_Output(&GPIOC_ODR,  _GPIO_PINMASK_2);     // Set LE2
  GPIO_Digital_Output(&GPIOA_ODR,  _GPIO_PINMASK_0);     // Set NS#
  GPIO_Digital_Output(&GPIOD_BASE, _GPIO_PINMASK_10);    // Set NS

  TLC5926_LE1 = 1;
  TLC5926_LE2 = 1;
  TLC5926_NS  = 1;
  TLC5926_nNS = 1;
  SPI3_Write(b);

  TLC5926_LE1 = 0;
  asm nop
  asm nop
  asm nop
  TLC5926_LE1 = 1;
  TLC5926_NS = 0;
  asm nop
  asm nop
  asm nop
  TLC5926_NS = 1;

  SPI3_Write(a);

  TLC5926_LE2 = 0;
  asm nop
  asm nop
  asm nop
  TLC5926_LE2 = 1;
  TLC5926_nNS = 0;
  asm nop
  asm nop
  asm nop
  TLC5926_nNS = 1;

  TLC5926_LE1 = 0;
  asm nop
  asm nop
  asm nop
  TLC5926_LE1 = 1;
  TLC5926_NS = 0;
  asm nop
  asm nop
  asm nop
  TLC5926_NS = 1;

  GPIO_Digital_Input(&GPIOD_IDR, _GPIO_PINMASK_10);
  PWM_TIM2_Start(_PWM_CHANNEL1, &_GPIO_MODULE_TIM2_CH1_PA0);
}

unsigned GetChar(char Character){
  if ((Character >= '-') && (Character <= '_'))
    return CharTable[Character-'-'];
  else
    return 0;
}

unsigned GetDigit(unsigned Digit){
  if ((Digit >= 0) && (Digit <= 9))
    return CharTable[Digit+3];
  else
    return 0;
}

void Intro() {
  WriteSegments(0xFFFF, 0xFFFF);
  Delay_ms(700);

  for(cnt=15;cnt>-1;cnt--){
    temp += ldexp(1, cnt);
    WriteSegments(temp,temp);
    Delay_ms(300);
    }
  delay_ms(1000);
}



// E X A M P L E S :
//-------------------------------------------------------------------------------
// C H A R A C T E R S
// WriteSegments(GetChar('E'), GetChar('K'));               // Write 'E' and 'K'
//-------------------------------------------------------------------------------
// D I G I T S
// WriteSegments(GetDigit(0), GetDigit(1));                 // Write '0' and '1'
//-------------------------------------------------------------------------------
// B I N A R Y  C O D E
// WriteSegments(0b0000000000000111, 0b0001000001100011);   // Write '7' and 'T'
//-------------------------------------------------------------------------------
// H E X  C O D E
// WriteSegments(0x104F, 0x0880);                           // Write 'D' and '/'
//-------------------------------------------------------------------------------


void WriteText() {
  WriteSegments(GetChar('M'), GetChar('I')); // Write M and I letters
  delay_ms(300);
  WriteSegments(GetChar('I'), GetChar('K')); // Write I and K letters
  delay_ms(300);
  WriteSegments(GetChar('K'), GetChar('R')); // Write K and R letters
  delay_ms(300);
  WriteSegments(GetChar('R'), GetChar('O')); // Write R and O letters
  delay_ms(300);
  WriteSegments(GetChar('O'), GetChar('E')); // Write O and E letters
  delay_ms(300);
  WriteSegments(GetChar('E'), GetChar('L')); // Write E and L letters
  delay_ms(300);
  WriteSegments(GetChar('L'), GetChar('E')); // Write L and E letters
  delay_ms(300);
  WriteSegments(GetChar('E'), GetChar('K')); // Write E and K letters
  delay_ms(300);
  WriteSegments(GetChar('K'), GetChar('T')); // Write K and T letters
  delay_ms(300);
  WriteSegments(GetChar('T'), GetChar('R')); // Write T and R letters
  delay_ms(300);
  WriteSegments(GetChar('R'), GetChar('O')); // Write R and O letters
  delay_ms(300);
  WriteSegments(GetChar('O'), GetChar('N')); // Write O and N letters
  delay_ms(300);
  WriteSegments(GetChar('N'), GetChar('I')); // Write N and I letters
  delay_ms(300);
  WriteSegments(GetChar('I'), GetChar('K')); // Write I and K letters
  delay_ms(300);
  WriteSegments(GetChar('K'), GetChar('A')); // Write K and A letters
}

void WriteSpecialChars(){
  delay_ms(3000);
  WriteSegments(0x2809, 0x136D);             // Write Sigma and $ characters
  delay_ms(3000);
  WriteSegments(0xFFFF, 0x3CC0);             // Write all segments and cyrilic letter
  delay_ms(3000);
  WriteSegments(0x0480, 0x2800);             // Write < and > characters
  delay_ms(3000);
}

void WriteNumbers(){
  delay_ms(1000);
  WriteSegments(GetDigit(0), GetDigit(1));   // Write 0 and 1 numbers
  delay_ms(3000);
  WriteSegments(GetDigit(1), GetDigit(2));   // Write 1 and 2 numbers
  delay_ms(1000);
  WriteSegments(GetDigit(2), GetDigit(3));   // Write 2 and 3 numbers
  delay_ms(1000);
}


void main() {
  unsigned char i = 0;
  unsigned char SPIN_Screen = 1;
  unsigned temp = 0;

  wait_time = 10;                                        // Set initial value for global variables
  current_duty = 0;

  GPIO_Digital_Output(&GPIOD_ODR, _GPIO_PINMASK_13);     // Set CS pin as output
  GPIO_Digital_Output(&GPIOC_ODR, _GPIO_PINMASK_2);      // Set RST pin as output
  GPIO_Digital_Output(&GPIOA_ODR, _GPIO_PINMASK_0);      // Set PWM pin as output
  GPIO_Digital_Output(&GPIOD_BASE, _GPIO_PINMASK_10);    // Set CS pin as output

  pwm_period1 = PWM_TIM2_Init(100);
  PWM_TIM2_Set_Duty(pwm_period1*0.5,  _PWM_NON_INVERTED, _PWM_CHANNEL1);  // Set current duty for PWM_TIM1
  // Inititalize SPI port
  SPI3_Init_Advanced(_SPI_FPCLK_DIV16, _SPI_MASTER | _SPI_16_BIT | _SPI_CLK_IDLE_LOW | _SPI_FIRST_CLK_EDGE_TRANSITION | _SPI_MSB_FIRST | _SPI_SS_DISABLE | _SPI_SSM_ENABLE | _SPI_SSI_1, &_GPIO_MODULE_SPI3_PC10_11_12);

  PWM_TIM1_Stop(_PWM_CHANNEL1);
  TLC5926_LE1 = 1;
  TLC5926_LE2 = 1;
  TLC5926_NS  = 1;
  TLC5926_nNS = 1;

  while(1){
    Intro();               // Display Intro
    WriteText();           // Display text MikroElektronika
    WriteSpecialChars();   // Display special characters
    WriteNumbers();        // Display numbers
  }

}